@@ -235,7 +235,7 @@ namespace Beagle.Util {
 				watching_for_updates = false;
 				fs = new FileStream (filepath, FileMode.Create);
 				XmlSerializer serializer = new XmlSerializer (type);
-				serializer.Serialize (fs, section);
+				XmlFu.SerializeUtf8 (serializer, fs, section);
 			} catch (Exception e) {
 				if (fs != null)
 					fs.Close ();
@@ -322,6 +322,81 @@ namespace Beagle.Util {
 				get { return show_search_window_binding; }
 				set { show_search_window_binding = value; }
 			}
+
+			private int max_displayed = 5;
+			public int MaxDisplayed {
+				get { return max_displayed; }
+				set {
+					if (value <= 0)
+						max_displayed = 1;
+					else
+						max_displayed = value;
+				}
+			}
+
+			// Best window position and dimension
+			// stored as percentage of screen co-ordinates
+			// to deal with change of resolution problem - hints from tberman
+
+			private float best_pos_x = 0;
+			public float BestPosX {
+				get { return best_pos_x; }
+				set { best_pos_x = value; }
+			}
+			
+			private float best_pos_y = 0;
+			public float BestPosY {
+				get { return best_pos_y; }
+				set { best_pos_y = value; }
+			}
+			
+			// dont explicitly set height on first run
+			private float best_width = 0; 
+			public float BestWidth {
+				get { return best_width; }
+				set { best_width = value; }
+			}
+
+			private float best_height = 0; // -ditto-
+			public float BestHeight {
+				get { return best_height; }
+				set { best_height = value; }
+			}
+
+			// FIXME Change names when Holmes is released under new name
+			// Holmes window position and dimension
+
+			private float holmes_pos_x = 0;
+			public float HolmesPosX {
+				get { return holmes_pos_x; }
+				set { holmes_pos_x = value; }
+			}
+			
+			private float holmes_pos_y = 0;
+			public float HolmesPosY {
+				get { return holmes_pos_y; }
+				set { holmes_pos_y = value; }
+			}
+			
+			private float holmes_width = 0; 
+			public float HolmesWidth {
+				get { return holmes_width; }
+				set { holmes_width = value; }
+			}
+
+			private float holmes_height = 0;
+			public float HolmesHeight {
+				get { return holmes_height; }
+				set { holmes_height = value; }
+			}
+
+			// ah!We want a Queue but Queue doesnt serialize *easily*
+			private ArrayList search_history = new ArrayList ();
+			public ArrayList SearchHistory {
+				get { return search_history; }
+				set { search_history = value; }
+			}
+
 		}
 
 		[ConfigSection (Name="daemon")]
@@ -392,7 +467,7 @@ namespace Beagle.Util {
 			[XmlArray]
 			[XmlArrayItem(ElementName="Root", Type=typeof(string))]
 			public ArrayList Roots {
-				get { return ArrayList.ReadOnly (roots); }
+				get { return roots; }
 				set { roots = value; }
 			}
 
@@ -406,7 +481,7 @@ namespace Beagle.Util {
 			[XmlArray]
 			[XmlArrayItem (ElementName="ExcludeItem", Type=typeof(ExcludeItem))]
 			public ArrayList Excludes {
-				get { return ArrayList.ReadOnly (excludes); }
+				get { return excludes; }
 				set { excludes = value; }
 			}
 
@@ -507,7 +582,7 @@ namespace Beagle.Util {
 			[XmlArray]
 			[XmlArrayItem(ElementName="PublicFolders", Type=typeof(string))]
 			public ArrayList PublicFolders {
-				get { return ArrayList.ReadOnly (publicFolders); }
+				get { return publicFolders; }
 				set { publicFolders = value; }
 			}
 
@@ -577,7 +652,7 @@ namespace Beagle.Util {
 			[XmlArray]
 			[XmlArrayItem(ElementName="NetBeagleNodes", Type=typeof(string))]
 			public ArrayList NetBeagleNodes {
-				get { return ArrayList.ReadOnly (netBeagleNodes); }
+				get { return netBeagleNodes; }
 				set { netBeagleNodes = value; }
 			}
 
