@@ -30,9 +30,10 @@ using System.Collections;
 using System.Diagnostics;
 using System.Threading;
 using Beagle.Util;
+
 namespace Beagle.Daemon {
 
-	public class QueryResult : IDisposable {
+	public class QueryResult : IQueryResult, IDisposable {
 
 		public delegate void StartedHandler (QueryResult source);
 		public event StartedHandler StartedEvent;
@@ -46,18 +47,15 @@ namespace Beagle.Daemon {
 		public delegate void FinishedHandler (QueryResult source);
 		public event FinishedHandler FinishedEvent;
 
-		public delegate void CancelledHandler (QueryResult source);
-		public event CancelledHandler CancelledEvent;
-
 		//////////////////////////////////
 
 		int workers = 0;
 		bool cancelled = false;
-		Hashtable hit_regulators = new Hashtable ();
 		Hashtable uri_hash = UriFu.NewHashtable ();
 		DateTime started_time;
 		DateTime finished_time;
 		Hashtable per_worker_started_time = new Hashtable ();
+		bool is_index_listener = false;
 
 
 		public QueryResult ()
@@ -86,18 +84,22 @@ namespace Beagle.Daemon {
 			get { return cancelled; }
 		}
 
+		public bool IsIndexListener {
+			get { return is_index_listener; }
+			set { is_index_listener = value; }
+		}
+
 		public void Cancel ()
 		{
 			lock (this) {
 				if (cancelled)
 					return;
 				cancelled = true;
-
-				if (CancelledEvent != null)
-					CancelledEvent (this);
 			}
 		}
 
+		// Note: some_hits is allowed to contain null.
+		// They are silently ignored.
 		public void Add (ICollection some_hits)
 		{
 			lock (this) {
@@ -108,15 +110,30 @@ namespace Beagle.Daemon {
 
 				if (some_hits.Count == 0)
 					return;
-		
-				foreach (Hit hit in some_hits)
-					uri_hash [hit.Uri] = hit;
+
+				if (IsIndexListener) {
+					if (HitsAddedEvent != null)
+						HitsAddedEvent (this, some_hits);
+					return;
+				}
+
+				// Be careful not to report the same hit twice.
+				ArrayList hits_to_report;
+				hits_to_report = new ArrayList ();
+				foreach (Hit hit in some_hits) {
+					if (hit != null && ! uri_hash.Contains (hit.Uri)) {
+						uri_hash [hit.Uri] = hit;
+						hits_to_report.Add (hit);
+					}
+				}
 				
-				if (HitsAddedEvent != null)
-					HitsAddedEvent (this, some_hits);
+				if (HitsAddedEvent != null && hits_to_report.Count > 0)
+					HitsAddedEvent (this, hits_to_report);
 			}
 		}
 
+		// Note: some_uris is allowed to contain null.
+		// They are silently ignored.
 		public void Subtract (ICollection some_uris)
 		{
 			lock (this) {
@@ -127,36 +144,30 @@ namespace Beagle.Daemon {
 
 				if (some_uris.Count == 0)
 					return;
+				
+				if (IsIndexListener) {
+					if (HitsSubtractedEvent != null)
+						HitsSubtractedEvent (this, some_uris);
+					return;
+				}
 
 				ArrayList filtered_uris = new ArrayList ();
 
 				// We only get to subtract a URI if it was previously added.
 				foreach (Uri uri in some_uris) {
-					if (uri_hash.Contains (uri)) {
+					if (uri != null && uri_hash.Contains (uri)) {
 						filtered_uris.Add (uri);
 						uri_hash.Remove (uri);
 					}
 				}
 
-				if (HitsSubtractedEvent != null)
+				if (HitsSubtractedEvent != null && filtered_uris.Count > 0)
 					HitsSubtractedEvent (this, filtered_uris);
 			}
 		}
 
 		//////////////////////////////////
 
-		public HitRegulator GetHitRegulator (Queryable queryable)
-		{
-			lock (hit_regulators) {
-				HitRegulator hr = hit_regulators [queryable] as HitRegulator;
-				if (hr == null) {
-					hr = new HitRegulator (queryable);
-					hit_regulators [queryable] = hr;
-				}
-				return hr;
-			}
-		}
-
 		// Given the Uri of a Hit contained in the QueryResult, return that Hit.
 		public Hit GetHitFromUri (Uri uri)
 		{
@@ -262,8 +273,8 @@ namespace Beagle.Daemon {
 
 				if (workers == 0) {
 					finished_time = now;
-					Logger.Log.Debug ("Last worker finished {0:0.00}s after start",
-							  (finished_time - started_time).TotalSeconds);
+					//Logger.Log.Debug ("Last worker finished {0:0.00}s after start",
+					//(finished_time - started_time).TotalSeconds);
 					if (FinishedEvent != null)
 						FinishedEvent (this);
 					Monitor.Pulse (this);
